function rm = GPregression(input,obs,kernel,options,obs_noise,obs1)
%GPregression Computes the standard GP regression model, contains a function for
%prediction and a function 

rm.input = input;
rm.kernel = kernel;

rm.dim = size(input,2);
rm.n = size(input,1);


%check data consistency
if (size(obs,2) ~= rm.n)
    disp('input data and observations have different dimesions');
    return;
end
if (kernel.dim ~= rm.dim)
    disp('kernel has the wrong dimensions');
    return;
end

if nargin <= 4
    obs_noise = options.noise_level;
end

    

%Process input, computing the necessary matrices
%compute gram matrix 

if kernel.derivative
    rm.N = rm.n*(1+rm.dim);
    rm.step = 1+rm.dim;
else
    rm.N = rm.n;
    rm.step = 1;
end

if length(obs_noise) == rm.n
    rm.noise = diag(obs_noise);   
else 
    rm.noise = eye(rm.N)*obs_noise; 
end


rm.K = zeros(rm.N);
for i = 1:rm.n
    I = 1 + (i-1)*rm.step;
    for j = 1:rm.n
        J = 1 + (j-1)*rm.step; 
        rm.K(I:(I+rm.step-1),J:(J+rm.step-1)) = kernel.evalGram(input(i,:),input(j,:),kernel);
    end
end
%adding the noise term
rm.K = rm.K + rm.noise;
rm.meanObs = mean(obs);

rm.L = chol(rm.K,'lower');
rm.Kinv = inv(rm.L')*inv(rm.L);
rm.conf = norminv((1+options.confidence_level)/2);

if kernel.derivative
    rm.y = zeros(rm.N,1);
    for i=1:rm.n
        I = 1 + (i-1)*rm.step;
        rm.y(I:(I+rm.step-1)) = [obs(i)-rm.meanObs;(obs1(:,i)-mean(obs1))'];
    end
else
    if isrow(obs)
        rm.y = (obs-rm.meanObs)';
    else
        rm.y = obs-rm.meanObs;
    end
end
rm.Kobs = rm.Kinv*rm.y;

rm.predict = @predict;
%rm.predictGradient = @predict;
rm.logmarginal = -rm.y'*rm.Kobs - sum(log(diag(rm.L)));


    function [p,l,u,sigma] = predict(x,rm) 
        k = zeros(rm.N,1);
        for jj=1:rm.n
           JJ = 1 + (jj-1)*rm.step;
           k(JJ:(JJ+rm.step-1)) = rm.kernel.evalPred(x,rm.input(jj,:),rm.kernel);
        end
        s = kernel.amplitude; 

        m_star = k'*rm.Kobs;
        %Kinvk = L'\(L\k);
        Kinvk = rm.Kinv*k;
        sigma2 = s - k'*Kinvk;

       if sigma2 < 0
           sigma2 = abs(sigma2);
       end
        
       sigma = sqrt(sigma2);
       pm = rm.conf*sigma;
       u = m_star + pm;
       l = m_star - pm;
       p = rm.meanObs + m_star; 
    end


end

