clear all;
N0 = 100;
modelSym = SIRsym(N0);
model = getModel(modelSym);
options = getOptions();

options.final_time = 300;
options.time_points = 50;
points = 5;
p = 2; %p=2 for SIRsym, p=1 for the other

input = linspace(model.par_range(p,1),model.par_range(p,2),points)';

range = [model.par_range(p,1),model.par_range(p,2);
         0,options.final_time];
gridPoints = [20,100];
grid = regularGrid(gridPoints,range);

[ time,obs,obs1 ] = simulateModel( model, input, options );
[INPUT,OBS,OBS1] = mergeTraj(input,time,obs,obs1);


%kernel = getKernel(2,1);
kernel = getKernel(2);
basisFunc =  @(X)[1, X(1), X(2), X(1)^2, X(1)*X(2), X(2)^2];
%basisFunc =  @(X)[1, X(1), X(2)];
bhdim = length(basisFunc([0,0]));

dBasisFunc = @(X)[0, 1, 0, 2*X(1), X(2),      0;
                  0, 0 ,1,      0, X(1), 2*X(2) ];

% scale = 5;
% for i=1:2
%     kernel.lengthscale(i) = (range(i,2) - range(i,1))/scale;
% end


%optimised ls
kernel.lengthscale(1) = 6.3003e-04;
kernel.lengthscale(2) = 18.9637;


% kernel.lengthscale(1) = 0.002;
% kernel.lengthscale(2) = 20;


gp = size(grid,1);
lower = ones(model.observables_dimension,1);
upper = ones(model.observables_dimension,1);
pred = zeros(model.observables_dimension,gp);
upperB = zeros(model.observables_dimension,gp);
lowerB = zeros(model.observables_dimension,gp);


for i=1:model.observables_dimension
    %kernel.lengthscale = kernel.estimateLSfunction(INPUT,OBS,OBS1,i,kernel);
    [l, u, p,lb,ub, bh, sh ] = GPreach(INPUT,i,OBS,OBS1,kernel,grid,options,basisFunc,dBasisFunc);
%    [l, u, p,lb,ub, bh, sh ] = GPreach1(INPUT,i,OBS,OBS1,kernel,grid,range,options,basisFunc,dBasisFunc);
  %  [ l, u, xl,xu, bh, sh ] = GPreachO(INPUT,i,OBS,OBS1,kernel,range,options,basisFunc,dBasisFunc);
    lower(i) = max(l,model.observable_range(i,1)*lower(i));
    upper(i) = min(u,model.observable_range(i,2)*upper(i));
    pred(i,:) = p;
    upperB(i,:) = ub;
    lowerB(i,:) = lb;
end


options.time_points = 100;
[ tt,nobs,~ ] = simulateModel( model, input, options );
u = reshape(nobs(1,1,:),options.time_points+1,1);
v = reshape(nobs(2,1,:),options.time_points+1,1);

%plot reachable polytope!
figure;
H = [ model.Jobs(zeros(kernel.dim,1));
     -model.Jobs(zeros(kernel.dim,1))];
K=[upper;-lower];
P = polytope(H,K);
V = extreme(P);
ind = convhull(V(:,1),V(:,2));
fill(V(ind,1),V(ind,2),'r');
hold on;
 

tp = options.time_points; %tp = options.time_points+1
M = points*(tp);
U = zeros(M,2);
for j=1:(tp)
    J = 1+(j-1)*points;
    U(J:(J+points-1),:) = [nobs(1,:,j)',nobs(2,:,j)'];
end
plot(U(:,1),U(:,2),'x');
 
% ind = convhull(V(:,1),V(:,2));
% V = V(ind,:);
%non-linear observable
% % % t = 0:0.001:2*pi;
% % % xc =  60+cos(t+pi)*sqrt(lower(5)); 
% % % yc = -10+sin(t+pi)*sqrt(lower(5));
% % % 
% % % W=zeros(2,0);
% % % for jj=1:length(xc)
% % %    if isinside(P,[xc(jj);yc(jj)])
% % %        W = [xc(jj),yc(jj);W];
% % %    end
% % % end
% % % 
% % % xc =  60+cos(t+pi)*sqrt(upper(5)); 
% % % yc = -10+sin(t+pi)*sqrt(upper(5));
% % % 
% % % Wu=zeros(2,0);
% % % for jj=1:length(xc)
% % %    if isinside(P,[xc(jj);yc(jj)])
% % %        Wu = [Wu;xc(jj),yc(jj)];
% % %    end
% % % end
% % % 
% % % 
% % % 
% % % VV = [Wu;V(2,:);W;V(3:(end-1),:);Wu(1,:)];
% % % fill(VV(:,1),VV(:,2),'r');



mU = mean(U);
C = princomp(U);

hold on;
x1 = 0:0.1:100;
for i=1:2
    y1 = (-1/C(2,i))*(C(1,i)*x1 - C(1,i)*mU(1) - C(2,i)*mU(2));
    plot(x1,y1,'r');
end

