clear all;
N0 = 100;
modelSym = SIRsym(N0);
model = getModel(modelSym);
options = getOptions();
GPUCBopt = GPUCBoptions();

TIME = 300;
p=[2,4];
model.explore_pars = zeros(model.parameter_dimension,1);
model.explore_pars(p) = 1;
range = model.par_range(p,:);

%construct template
gridPoints = [5,5];
grid = regularGrid(gridPoints,range);
options.final_time = TIME;
options.time_points = 1;
[ ~,obs,~ ] = simulateModel( model, grid, options );
U = (obs(:,:,end))';
C = princomp(U);
H0 = [1,0;
      0,1;
      1,1];
H1 = C';  
vb = [0,N0];
H0_bounds = [0,N0;
             0,N0;
             0,N0];
H1_bounds = [min(H1(:,1)*vb(1),H1(:,1)*vb(2))+min(H1(:,2)*vb(1),H1(:,2)*vb(2)),...
    max(H1(:,1)*vb(1),H1(:,1)*vb(2))+max(H1(:,2)*vb(1),H1(:,2)*vb(2))];
H = [H0;H1];
H_bounds = [H0_bounds;H1_bounds];

%get kernel
bf =  @(X)[1, X(1), X(2), X(1)^2, X(1)*X(2), X(2)^2];
dbf = @(X)[0, 1, 0, 2*X(1), X(2),      0;
                  0, 0 ,1,      0, X(1), 2*X(2) ];
kernel = getKernel(2,bf,dbf);
kernel.lengthscale(1) = 6.3003e-04;
kernel.lengthscale(2) = 0.02;
kernel.confidence_level = 0.95;


%set GPUCB options
  %grid sampling
    GPUCBopt.gridSampling = 1;
    GPUCBopt.randomInitialPoints = 0;
    GPUCBopt.othogonalSampleBins = 10;
    GPUCBopt.orthogonalSamplePointsPerBin = 3;
  %initial points
    GPUCBopt.randomInitialPoints = 0;
    GPUCBopt.initialObs = [5,5]; %25 initial points in a grid!
  %other options - see if for the moment it makes sense
    GPUCBopt.hyperparameterOptimisation=0;
    GPUCBopt.extraGrid = 0;
    GPUCBopt.debug = 0;

    
  h = size(H,1);  
  lower = ones(h,1);
  upper = ones(h,1);
  lower_P = ones(h,1);
  upper_P = ones(h,1);

  for i=1:h
       [l,u,pl,pu] = GPUCB_reach_T( model, TIME, range,H(i,:), kernel, options, GPUCBopt);
       lower(i) = max(l,H_bounds(i,1));
       upper(i) = min(u,H_bounds(i,2));
       lower_P(i) = pl;
       upper_P(i) = pu;
  end


%plot reachable polytope!
figure;
HH = [H;-H];
K=[upper;-lower];
P = polytope(HH,K);
V = extreme(P);
ind = convhull(V(:,1),V(:,2));
fill(V(ind,1),V(ind,2),'r');
hold on;
plot(U(:,1),U(:,2),'x');

