clear all;
N0 = 1000;
init_java;
import simhya.matlab.SimHyAModel;

model_file = 'model/SIR.txt';

m = SimHyAModel();
m.loadModel(model_file);
m.setSSA();

options = getOptions();
GPUCBopt = GPUCBoptions();

TIME = 300;
par_names = {'ki','kr'};
range = [0.05/N0,0.15/N0;
         0.01,0.1];
q = 0.95;     

stat = @(X) stat_average(X);

%statU = stat;
%statL = stat;
%options.bootstrap = false;

statU = @(X) stat_quantile(X,(1+q)/2);
statL = @(X) stat_quantile(X,(1-q)/2);
options.bootstrap = true;

%construct template
gridPoints = [5,5];
grid = regularGrid(gridPoints,range);
options.stoch_runs = 1000;
U = zeros(size(grid,1),2);
for j = 1:size(grid,1)
   U(j,1) = GPUCB_stochT( m, TIME, grid(j,:), par_names, [1,0], stat, options );
   U(j,2) = GPUCB_stochT( m, TIME, grid(j,:), par_names, [0,1], stat, options );
end
C = princomp(U);
H0 = [1,0;
      0,1;
      1,1];
H1 = C';  
vb = [0,N0];
H0_bounds = [0,N0;
             0,N0;
             0,N0];
H1_bounds = [min(H1(:,1)*vb(1),H1(:,1)*vb(2))+min(H1(:,2)*vb(1),H1(:,2)*vb(2)),...
    max(H1(:,1)*vb(1),H1(:,1)*vb(2))+max(H1(:,2)*vb(1),H1(:,2)*vb(2))];
H = [H0;H1];
H_bounds = [H0_bounds;H1_bounds];



%get kernel
bf =  @(X)[1, X(1), X(2), X(1)^2, X(1)*X(2), X(2)^2];
dbf = @(X)[0, 1, 0, 2*X(1), X(2),      0;
                  0, 0 ,1,      0, X(1), 2*X(2) ];
kernel = getKernel(2,bf,dbf);
kernel.lengthscale(1) = 6.3003e-04;
kernel.lengthscale(2) = 0.02;
kernel.confidence_level = 0.95;


%set GPUCB options
  %grid sampling
    options.stoch_runs = 200;
    GPUCBopt.gridSampling = 1;
    GPUCBopt.randomInitialPoints = 0;
    GPUCBopt.othogonalSampleBins = 10;
    GPUCBopt.orthogonalSamplePointsPerBin = 3;
  %initial points
    GPUCBopt.randomInitialPoints = 0;
    GPUCBopt.initialObs = [5,5];
    GPUCBopt.orthogonalSamplingInitialPoints=1;
    GPUCBopt.orthogonalSamplePointsPerBinInit = 2;%32 points to start
    GPUCBopt.othogonalSampleBinsInit = 4;
  %other options - see if for the moment it makes sense
    GPUCBopt.hyperparameterOptimisation=0;
    GPUCBopt.extraGrid = 0;
    GPUCBopt.debug = 0;

    
  h = size(H,1);  
  lower = ones(h,1);
  upper = ones(h,1);
  lower_P = ones(h,1);
  upper_P = ones(h,1);

  for i=1:h
       [u,pu] = GPUCB_stochreach_T(m, TIME, par_names, range, H(i,:), statU, kernel, options, true,  GPUCBopt);
       [l,pl] = GPUCB_stochreach_T(m, TIME, par_names, range, H(i,:), statL, kernel, options, false, GPUCBopt);
       lower(i) = max(l,H_bounds(i,1));
       upper(i) = min(u,H_bounds(i,2));
       lower_P(i) = pl;
       upper_P(i) = pu;
  end


%plot reachable polytope!
figure;
HH = [H;-H];
K=[upper;-lower];
P = polytope(HH,K/N0);
V = extreme(P);
ind = convhull(V(:,1),V(:,2));
fill(V(ind,1),V(ind,2),'r');
hold on;
plot(U(:,1),U(:,2),'x');

sss = 500;
W = zeros(0,2);
for j = 1:size(grid,1)
    m.setParameters(par_names, grid(j,:));
    xx = m.simulateAtT(TIME,sss);
    W = [W;xx(:,2:3)];
end    
plot(W(:,1),W(:,2),'x');

c=0;
for j=1:size(W,1)
    c = c + isinside(P,W(j,:)');
end
c = c/size(W,1);
