function [amplitude,lengthscale] = GPUCB_optimiseHyperparameters(inputs,obs,noise,kernel,trials)
%propCheckOptimiseHyperparameters Optimises kernel hyperparameters (only
%for the gaussian kernel)
%   
if nargin <= 4
    trials = 10;
end
n = size(inputs,2);
obsn=obs-mean(obs);
lower = [1e-5;kernel.RBFminLengthScale]; %amplitude and lengthscale
upper = [(max(obs)-min(obs));kernel.RBFmaxLengthScale];
options = optimset('Display','off','Algorithm','active-set');

mm = -Inf;
amplitude = 0; 
lengthscale = 0;

% using a stupid multiple restart strategy
for i=1:trials
    point = lower + rand(2,1).*(upper-lower);
    [hmax,fval,~,~,~,~,~] = fmincon(@log_marginal,point,[],[],[],[],...
       lower,upper,[],options);
   if fval > mm
      amplitude = hmax(1); 
      lengthscale = hmax(2); 
   end
end



    %computes minus the log marginal likelihood for a given set of
    %hyperparameters
    function lm = log_marginal(hyperp)
       old_a = kernel.RBFamplitude; 
       old_l = kernel.RBFlengthScale;
       kernel.RBFamplitude = hyperp(1);
       kernel.RBFlengthScale = hyperp(2);
       
       K = zeros(n);
        for l=1:n
           for k=l:n
               K(l,k) =...
                   kernel.function(inputs(:,l),inputs(:,k),kernel);
               K(k,l) = K(l,k);
           end
        end 
       K = K + diag(noise);
       L = chol(K,'lower');
       aux = L'\(L\obsn');
       lm = 0.5*obsn*aux + sum(log(diag(L))) + n/2*log(2*pi); 
       kernel.RBFamplitude = old_a;
       kernel.RBFlengthScale = old_l;
    end


end

