function rm = GPregression(input,obs,noise,kernel)
%GPregression Computes the standard GP regression model, contains a function for
%prediction and a function 

rm.input = input;
rm.kernel = kernel;

rm.dim = size(input,2);
rm.n = size(input,1);


%check data consistency
if (size(obs,2) ~= rm.n)
    disp('input data and observations have different dimesions');
    return;
end
if (kernel.dim ~= rm.dim)
    disp('kernel has the wrong dimensions');
    return;
end

if length(noise) == 1
    rm.noise = noise*ones(rm.n,1);
else
    rm.noise = noise;
end

%Process input, computing the necessary matrices
%compute gram matrix and H matrix

rm.selfvariance = kernel.amplitude;

    

%Process input, computing the necessary matrices
%compute gram matrix 

rm.K = zeros(rm.n);
for i = 1:rm.n
    for j = 1:rm.n
        rm.K(i,j) = kernel.evalGram(input(i,:),input(j,:),kernel);
    end
end
%adding the noise term
rm.K = rm.K + diag(rm.noise.^2);
rm.meanObs = mean(obs);

rm.L = chol(rm.K,'lower');
rm.Kinv = inv(rm.L')*inv(rm.L);
rm.conf = norminv((1+kernel.confidence_level)/2);
if isrow(obs)
    rm.y = (obs-rm.meanObs)';
else
    rm.y = obs-rm.meanObs;
end
rm.Kobs = rm.Kinv*rm.y;

rm.predict = @predict;
rm.logmarginal = -rm.y'*rm.Kobs - sum(log(diag(rm.L)));


    function [p,sigma,l,u] = predict(x,rm) 
        k = zeros(rm.N,1);
        for jj=1:rm.n
            k(jj) = rm.kernel.evalPred(x,rm.input(jj,:),rm.kernel);
        end
        s = rm.selfvariance; 

        m_star = k'*rm.Kobs;
        %Kinvk = L'\(L\k);
        Kinvk = rm.Kinv*k;
        sigma2 = s - k'*Kinvk;

       if sigma2 < 0
           sigma2 = abs(sigma2);
       end
        
       sigma = sqrt(sigma2);
       pm = rm.conf*sigma;
       u = rm.meanObs + m_star + pm;
       l = rm.meanObs + m_star - pm;
       p = rm.meanObs + m_star; 
    end


end

